@@ -19,13 +19,15 @@
 Provides classes and functions to read and write gbs.conf.
 '''
 
-from __future__ import with_statement
+
 import os
 import re
 import base64
+import binascii
+import codecs
 import shutil
 from collections import namedtuple
-from ConfigParser import SafeConfigParser,  \
+from configparser import ConfigParser,  \
                          MissingSectionHeaderError, Error
 
 from gitbuildsys import errors
@@ -35,22 +37,22 @@ from gitbuildsys.log import LOGGER as log
 
 def decode_passwdx(passwdx):
     '''decode passwdx into plain format'''
-    return base64.b64decode(passwdx).decode('bz2')
+    return codecs.decode(base64.b64decode(passwdx.encode()), 'bz2').decode()
 
 
 def encode_passwd(passwd):
     '''encode passwd by bz2 and base64'''
-    return base64.b64encode(passwd.encode('bz2'))
+    return base64.b64encode(codecs.encode(passwd.encode(), 'bz2')).decode()
 
 
-class BrainConfigParser(SafeConfigParser):
+class BrainConfigParser(ConfigParser):
     """Standard ConfigParser derived class which can reserve most of the
     comments, indents, and other user customized stuff inside the ini file.
     """
 
     def read_one(self, filename):
         """only support one input file"""
-        return SafeConfigParser.read(self, filename)
+        return ConfigParser.read(self, filename)
 
     def _read(self, fptr, fname):
         """Parse a sectioned setup file.
@@ -65,7 +67,7 @@ class BrainConfigParser(SafeConfigParser):
         self._flines = fptr.readlines()
         fptr.seek(0)
 
-        return SafeConfigParser._read(self, fptr, fname)
+        return ConfigParser._read(self, fptr, fname)
 
     def _set_into_file(self, section, option, value, replace_opt=None):
         """Set the value in the file contents
@@ -145,9 +147,9 @@ class BrainConfigParser(SafeConfigParser):
         """
         if not self.has_section(section):
             self.add_section(section)
-        SafeConfigParser.set(self, section, option, value)
+        ConfigParser.set(self, section, option, value)
         if replace_opt:
-            SafeConfigParser.remove_option(self, section, replace_opt)
+            ConfigParser.remove_option(self, section, replace_opt)
 
         # If the code reach here, it means the section and key are ok
         try:
@@ -257,9 +259,9 @@ url = http://download.tizen.org/releases/daily/trunk/ivi/latest/
     def _create_default_parser(self):
         'create a default parser that handle DEFAULTS values'
         parser = BrainConfigParser()
-        for sec, options in self.DEFAULTS.iteritems():
+        for sec, options in list(self.DEFAULTS.items()):
             parser.add_section(sec)
-            for key, val in options.iteritems():
+            for key, val in list(options.items()):
                 parser.set(sec, key, val)
         return parser
 
@@ -268,7 +270,7 @@ url = http://download.tizen.org/releases/daily/trunk/ivi/latest/
 
         self._cfgparsers = []
         for fpath in self._cfgfiles:
-            cfgparser = BrainConfigParser()
+            cfgparser = BrainConfigParser(strict=False)
             try:
                 cfgparser.read_one(fpath)
                 if cfgparser.has_section('general') and \
@@ -346,7 +348,9 @@ url = http://download.tizen.org/releases/daily/trunk/ivi/latest/
 
     def _check_passwd(self):
         'convert passwd item to passwdx and then update origin conf files'
-        dirty = set()
+        #For python3.x set() to call add(), it will show error: TypeError: unhashable type: 'BrainConfigParser'
+        #This is because for set() in python3.x it can only add which can be hashed, such as int, float, tuple, str
+        dirty = []
 
         all_sections = set()
         for layer in self._cfgparsers:
@@ -368,7 +372,8 @@ url = http://download.tizen.org/releases/daily/trunk/ivi/latest/
                                                     key + 'x',
                                                     encode_passwd(plainpass),
                                                     key)
-                            dirty.add(cfgparser)
+                            if (cfgparser not in dirty):
+                                dirty.append(cfgparser)
 
         if dirty:
             log.warning('plaintext password in config files will '
@@ -377,26 +382,34 @@ url = http://download.tizen.org/releases/daily/trunk/ivi/latest/
 
     def _get(self, opt, section='general'):
         'get value from multi-levels of config file'
+        sect_found = True
+        gerr = None
         for cfgparser in self._cfgparsers:
             try:
                 return cfgparser.get(section, opt)
             except Error as err:
+                sect_found = False
+                gerr = err
                 pass
-        raise errors.ConfigError(err)
+
+        if not sect_found:
+            raise errors.ConfigError(gerr)
 
     def options(self, section='general'):
         'merge and return options of certain section from multi-levels'
         sect_found = False
+        gerr = None
         options = set()
         for cfgparser in self._cfgparsers:
             try:
                 options.update(cfgparser.options(section))
                 sect_found = True
             except Error as err:
+                gerr = err
                 pass
 
         if not sect_found:
-            raise errors.ConfigError(err)
+            raise errors.ConfigError(gerr)
 
         return options
 
@@ -431,7 +444,8 @@ url = http://download.tizen.org/releases/daily/trunk/ivi/latest/
                  val = self._get('passwdx', section)
                  try:
                      ret = decode_passwdx(val)
-                 except (TypeError, IOError) as err:
+                 #for bad passwdx, its error type is binascii.Error for python3.x
+                 except (binascii.Error, TypeError, IOError) as err:
                      raise errors.ConfigError('passwdx:%s' % err)
         else:
             ret = self._get(opt, section)
@@ -599,7 +613,7 @@ class BizConfigManager(ConfigMgr):
         except KeyError as err:
             raise errors.ConfigError('unknown key: %s. Supportted '\
                     'keys are %s' % (str(err), ' '.join( \
-                    self.DEFAULTS['general'].keys())))
+                    list(self.DEFAULTS['general'].keys()))))
         return value
 
     def is_profile_oriented(self):
@@ -766,7 +780,7 @@ class BizConfigManager(ConfigMgr):
                     repos[key]['passwd'] = value
                 else:
                     repos[key][name] = value
-        return sorted(repos.items(), key=lambda i: i[0])
+        return sorted(list(repos.items()), key=lambda i: i[0])
 
     def _build_profile_by_subcommand(self):
         '''return profile object from subcommand oriented style of config'''
@@ -798,9 +812,9 @@ class BizConfigManager(ConfigMgr):
 
         return profile
 
-class MappingConfigParser(SafeConfigParser):
+class MappingConfigParser(ConfigParser):
     def __init__(self, file, defaults=None):
-        SafeConfigParser.__init__(self, defaults=None)
+        ConfigParser.__init__(self, defaults=None)
         self._obs_meta = {}
         self._prefix_meta = {}
         self._repo_meta = {}
